/*
 * The Library class implements a highly simplified and limited model of 
 * a library. LibraryTest class executes some tests on Library.
 * 
 */

class Library {


	/*
	 * The constructor initialize the data structures of Library class
	 * The data consists of three arrays. 
	 * 
	 * bookISBN - int[]
	 * 
	 * "bookISBN" stores ISBN numbers of books (an integer in this task).
	 * For simplicity, It is assumed that all books in bookISBN are available
	 * 
	 * memberID - int[] 
	 * 
	 * The array "memberID" stores member IDs (int)
	 * 
	 * borrowed - boolean[] 
	 * 
	 * "borrowed" is a boolean array which records if a member has borrowed a book.
	 * If the ID of a member is stored in the array index k of the memberID array,
	 * then the borrowed status of the member is in borrowed[k]. If it is true then
	 * the member has already borrowed a book.
	 * 
	 */
	
	private static final int maxBook = 10;
	private static final int maxMember = 10;
	private static final int initBookCount = 3;
	private static final int initMemberCount = 3;
	
	public Library() {
		
		bookISBN = new int[maxBook];
		memberID = new int[maxMember];
		borrowed = new boolean[maxMember];
		
		/*
		 * The initialize function adds 3 books and 3 members to the library.
		 */
		initialize();
	}

	/*
	 * The function accepts the ID of the member as argument and stores it in 
	 * the memberID array. 
	 * 
	 * Pre-Condition -
	 * 	Member ID must not be zero.
	 * 	Member space must not be full.
	 * 	Member must not already exist.
	 * 
	 * Post-Condition -
	 * 	Member data is stored in an empty cell.
	 *  Data of other members must not changed
	 */

    /*@ normal_behaviour
    @
    @
    @
    @
    @
    @   assignable memberID[*];
    @*/
	public void addMember(int mid) {
		memberID[find(memberID,0)] = mid;
	}

	/*
	 * Adds a books to the library. The function accept ISBN number
	 * of the book.
	 */

	
	public void addBook(int isbn) {
		bookISBN[find(bookISBN,0)] = isbn;
	}

	/*
	 * The function accepts member ID, and book ISBN as arguments
	 * and marks the book as borrowed.
	 * 
	 * Pre-Condition -
	 * 	Member ID must not be zero.
	 * 	Book isbn must not be zero.
	 * 	Member must exist.
	 * 	Book must be available.
	 * 	Member must not have borrowed any book already.
	 * 
	 * 
	 * Post-Condition -
	 * 	The borrow status of member is marked as true
	 * 	The status of other member must not change.
	 *  
	 */
	
	/*@ normal_behaviour
    @   
    @  
    @  
    @  
    @  
    @   assignable borrowed[*];
    @*/
	
	public void borrowbook(int mid, int isbn) {
		borrowed[find(memberID,mid)] = true;
	}


	protected void initialize() {

		// Add books to Library
		for(int i = 0; i < initBookCount ; i++) {
			bookISBN[i] = i+1;
		}

		//Add members to Library
		for(Integer i = 0 ; i < initMemberCount ; i++) {
			memberID[i] = i+1;
		}
			
	}
	
	/*
	 * The function finds a number (key) in an int array. 
	 * It returns the length of the array if key is not found
	 * else returns the index of key.
	 * 
	 * Pre-Condition -
	 * 	Array must not be null.
	 * 	Array must not be empty.
	 * 
	 * Post-Condition -
	 * 	Return value must not be greater than array size
	 * 	Return array size if the key is not found
	 * 	Return array index of the key if it is found.
	 */

    /*@ normal_behaviour
    @   
    @   
    @   
    @	
    @	assignable \nothing;
    @*/
	protected int find(int[] a, int key) {
		
		int i = 0;
		
        /*@ loop_invariant
        @  // loop invariant
        @  assignable i;
        @  // loop variant
        @*/
		while(i < a.length && a[i] != key)
			i++;
		
		return i;
	}
	

	private /*@ spec_public @*/ int[] bookISBN;
	private /*@ spec_public @*/ int[] memberID;
	private /*@ spec_public @*/ boolean[] borrowed;
	
	
	public static void main(String[] args) {

	}
}

